/* Copyright 2007, Unpublished Work of Technologic Systems
 * All Rights Reserved.
 *
 * THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
 * PROPRIETARY AND TRADE SECRET INFORMATION OF TECHNOLOGIC SYSTEMS.
 * ACCESS TO THIS WORK IS RESTRICTED TO (I) TECHNOLOGIC SYSTEMS 
 * EMPLOYEES WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE
 * OF THEIR ASSIGNMENTS AND (II) ENTITIES OTHER THAN TECHNOLOGIC
 * SYSTEMS WHO HAVE ENTERED INTO APPROPRIATE LICENSE AGREEMENTS.  NO
 * PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED, 
 * DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, 
 * EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED, ADAPTED
 * IN ANY FORM OR BY ANY MEANS, MANUAL, MECHANICAL, CHEMICAL, 
 * ELECTRICAL, ELECTRONIC, OPTICAL, BIOLOGICAL, OR OTHERWISE WITHOUT
 * THE PRIOR WRITTEN PERMISSION AND CONSENT OF TECHNOLOGIC SYSTEMS.
 * ANY USE OR EXPLOITATION OF THIS WORK WITHOUT THE PRIOR WRITTEN
 * CONSENT OF TECHNOLOGIC SYSTEMS COULD SUBJECT THE PERPETRATOR TO
 * CRIMINAL AND CIVIL LIABILITY.
 */

module irq_controller(
  wb_clk_i,
  wb_rst_i,

  wbs_cyc_i,
  wbs_stb_i,
  wbs_adr_i,
  wbs_dat_i,
  wbs_dat_o,
  wbs_ack_o,
  wbs_we_i,
  wbs_sel_i,

  wbm_cyc_o,
  wbm_stb_o,
  wbm_adr_o,
  wbm_dat_o,
  wbm_dat_i,
  wbm_ack_i,
  wbm_we_o,
  wbm_sel_o,

  irq_i,
  irq_o

);
input wb_clk_i, wb_rst_i;
input wbs_cyc_i, wbs_stb_i, wbs_we_i;
output wbs_ack_o;
output [31:0] wbs_dat_o;
input [31:0] wbs_adr_i, wbs_dat_i;
input [3:0] wbs_sel_i;

output wbm_cyc_o, wbm_stb_o, wbm_we_o;
input wbm_ack_i;
output [31:0] wbm_adr_o, wbm_dat_o;
input [31:0] wbm_dat_i;
output [3:0] wbm_sel_o;

input [31:0] irq_i;
output irq_o;

reg [31:0] msi_adr, msi_msk, wbm_dat, wbs_dat;
reg [31:0] irq_q;
reg wbm_cyc, wbs_ack, msi_ack;
assign wbm_we_o = 1'b1;
assign wbm_sel_o = 4'b1111;
assign wbm_adr_o = msi_adr;
assign wbm_dat_o = irq_q;
assign wbm_cyc_o = wbm_cyc;
assign wbm_stb_o = wbm_cyc;
assign wbs_ack_o = wbs_ack;
assign wbs_dat_o = wbs_dat;
assign irq_o = ((irq_i & (~msi_msk)) != 32'h0);
always @(posedge wb_clk_i or posedge wb_rst_i) begin
  if (wb_rst_i) begin
    msi_adr <= 32'h0;
    msi_msk <= 32'hffffffff;
    irq_q <= 32'h0;
    wbm_cyc <= 1'b0;
    wbs_ack <= 1'b0;
    wbs_dat <= 32'h0;
    msi_ack <= 1'b0;
  end else begin
    wbs_ack <= wbs_cyc_i && wbs_stb_i && !wbs_ack && !msi_ack;

    if (wbm_cyc) begin
      if (wbm_ack_i) begin
        wbm_cyc <= 1'b0;
      end
    end else if (((irq_i ^ irq_q) & irq_i & msi_msk) != 32'h0) begin
      wbm_cyc <= 1'b1;
      irq_q <= irq_i & msi_msk;
      msi_ack <= 1'b0;
    end else begin
      if (msi_ack) irq_q <= 32'd0; 
      else irq_q <= irq_i & msi_msk;
      msi_ack <= 1'b0;      
    end

    if (wbs_cyc_i && wbs_stb_i && wbs_we_i && !wbs_ack && 
      !msi_ack) begin
      msi_ack <= 1'b1;
      case (wbs_adr_i[3:2])
      2'd0: msi_adr <= {16'd0, wbs_dat_i[15:0]};
      2'd1, 2'd2, 2'd3: msi_msk <= {16'd0, wbs_dat_i[15:0]}; // 1'b1 - MSI enabled
      endcase
    end

    case (wbs_adr_i[3:2])
      2'd0: wbs_dat <= msi_adr;
      2'd1: wbs_dat <= msi_msk;
	  2'd2: wbs_dat <= irq_i & (~msi_msk); 
	  2'd3: wbs_dat <= irq_i;
    endcase
  end
end

endmodule

